@@ -27,3 +27,5 @@ program Obj = ((program)"obj");
 #define CTX_RETURN     6
 #define CTX_MAPPING    7
 #define CTX_CALL_ASYNC 8
+#define CTX_EXISTS  9
+#define CTX_EXIST_RES  10
