@@ -1,5 +1,5 @@
 /*
- * $Id: acconfig.h,v 1.38 1999/05/26 03:08:21 mast Exp $
+ * $Id: acconfig.h,v 1.79 2001/07/13 18:21:30 grubba Exp $
  */
 #ifndef MACHINE_H
 #define MACHINE_H
@@ -15,9 +15,50 @@
 /* Define this if you want run time self tests */
 #undef PIKE_DEBUG
 
+/* Define this if you are going to use a memory access checker (like Purify) */
+#undef __CHECKER__
+
 /* Define this if you want malloc debugging */
 #undef DEBUG_MALLOC
 
+/* Define this to enable the internal Pike security system */
+#undef PIKE_SECURITY
+
+/* Define this to enable the internal bignum conversion */
+#undef AUTO_BIGNUM
+
+/* Define this if you want to enable the shared nodes mode of the optimizer. */
+#undef SHARED_NODES
+
+/* The following USE_* are used by smartlink */
+/* Define this if your ld sets the run path with -rpath */
+#undef USE_RPATH
+
+/* Define this if your ld sets the run path with -R */
+#undef USE_R
+
+/* Define this if your ld sets the run path with -YP, */
+#undef USE_YP_
+
+/* Define this if your ld sets the run path with +b */
+#undef USE_PLUS_b
+
+/* Define this if your ld uses -rpath, but your cc wants -Wl,-rpath, */
+#undef USE_Wl
+
+/* Define this if your ld uses -R, but your cc wants -Wl,-R */
+#undef USE_Wl_R
+
+/* Define this if your ld uses -YP, , but your cc wants -Xlinker -YP, */
+#undef USE_XLINKER_YP_
+
+/* Define this if your ld doesn't have an option to set the run path */
+#undef USE_LD_LIBRARY_PATH
+
+/* Define this if your compiler attempts to use _chkstk, but libc contains
+ * __chkstk. */
+#undef HAVE_BROKEN_CHKSTK
+
 @TOP@
 
 /* Enable profiling */
@@ -32,6 +73,27 @@
 /* Define to 'int' if <sys/time.h> doesn't */
 #undef time_t
 
+/* Define to 'short' if <sys/types.h> doesn't */
+#undef pri_t
+
+/* Define to 'int' if <sys/types.h> doesn't */
+#undef uid_t
+
+/* Define to 'int' if <sys/types.h> doesn't */
+#undef gid_t
+
+/* Define to 'int' if <sys/types.h> doesn't */
+#undef pid_t
+
+/* Define to 'unsigned long' if <sys/types.h> or <stddef.h> doesn't */
+#undef size_t
+
+/* Define to 'long' if <sys/types.h> of <stddef.h> doesn't */
+#undef ptrdiff_t
+
+/* Define to 'long' if <sys/types.h> doesn't */
+#undef off_t
+
 /* Define to 'int' if <signal.h> doesn't */
 #undef sig_atomic_t
 
@@ -41,6 +103,12 @@
 /* define this if igonoring SIGFPE helps with core dumps */
 #undef IGNORE_SIGFPE
 
+/* define if you want to use double precision floats instead of single */
+#undef WITH_DOUBLE_PRECISION_SVALUE
+
+/* define if you want to use long double precision floats */
+#undef WITH_LONG_DOUBLE_PRECISION_SVALUE
+
 /* If using the C implementation of alloca, define if you know the
  * direction of stack growth for your system; otherwise it will be
  * automatically deduced at run-time.
@@ -52,12 +120,27 @@
  */
 #undef STACK_DIRECTION
 
+/* Define this to the number of KB in the initial stack,
+ * currently this is 1 Mb on FreeBSD, 2Mb on Linux and
+ * unlimited (undefined) everywhere else
+ */
+#undef Pike_INITIAL_STACK_SIZE
+
 /* If so, is it restricted to user and system time? */
 #undef GETRUSAGE_RESTRICTED
 
 /* Solaris has rusage as an ioctl on procfs */
 #undef GETRUSAGE_THROUGH_PROCFS
 
+/* So has True64, but no useful information in prstatus_t */
+#undef GETRUSAGE_THROUGH_PROCFS_PRS
+
+/* Define if you have infnan */
+#undef HAVE_INFNAN
+
+/* Define if you have _isnan */
+#undef HAVE__ISNAN
+
 /* Define if you have fork */
 #undef HAVE_FORK
 
@@ -67,6 +150,12 @@
 /* Define if you have fpsetmask */
 #undef HAVE_FPSETMASK
 
+/* Define if you have fpsetround */
+#undef HAVE_FPSETROUND
+
+/* Define if you have isless */
+#undef HAVE_ISLESS
+
 /* Define if you have crypt.  */
 #undef HAVE_CRYPT
 
@@ -79,6 +168,15 @@
 /* Define if gettimeofday takes to arguments */
 #undef GETTIMEOFDAY_TAKES_TWO_ARGS
 
+/* Define if you have gethrtime */
+#undef HAVE_GETHRTIME
+
+/* Can we make our own gethrtime? */
+#undef OWN_GETHRTIME
+
+/* ... by using the RDTSC instruction? */
+#undef OWN_GETHRTIME_RDTSC
+
 /* Define if you have a working, 8-bit-clean memcmp */
 #undef HAVE_MEMCMP
 
@@ -93,27 +191,42 @@
 /* Define if you have memmem.  */
 #undef HAVE_MEMMEM
 
+/* Define if you have memset.  */
+#undef HAVE_MEMSET
+
+/* Define if you have memcpy.  */
+#undef HAVE_MEMCPY
+
+/* Define if you have strcoll */
+#undef HAVE_STRCOLL
+
 /* Define this if you have dlopen */
 #undef HAVE_DLOPEN
 
 /* Define if you have ldexp.  */
 #undef HAVE_LDEXP
 
+/* Define if you have rint.  */
+#undef HAVE_RINT
+
 /* Define if you have frexp.  */
 #undef HAVE_FREXP
 
 /* Define if your signals are one-shot */
 #undef SIGNAL_ONESHOT
 
-/* Define if you don't have the pri_t typedef. */
-#undef pri_t
-
-/* You have gcc stype function attributes? */
+/* You have gcc-type function attributes? */
 #undef HAVE_FUNCTION_ATTRIBUTES
 
+/* You have lc-type __declspec? */
+#undef HAVE_DECLSPEC
+
 /* Do your compiler grock 'volatile' */
 #define VOLATILE volatile
 
+/* Define this if your compiler doesn't allow cast of void * to function pointer */
+#undef NO_CAST_TO_FUN
+
 /* How to extract a char and an unsigned char from a char * */
 #undef EXTRACT_CHAR_BY_CAST
 #undef EXTRACT_UCHAR_BY_CAST
@@ -147,12 +260,18 @@
 /* define if declaration of getenv is missing */
 #undef GETENV_DECL_MISSING
 
+/* define if you are using crypt.c. */
+#undef USE_CRYPT_C
+
 /* Define if we can declare 'extern char **environ' */
 #undef DECLARE_ENVIRON
 
 /* What byteorder does your machie use most machines use 4321, PC use 1234 */
 #define PIKE_BYTEORDER 0
 
+/* What alignment do 32-bit integers need */
+#define PIKE_INT32_ALIGNMENT 4
+
 /* Number of possible filedesriptors */
 #define MAX_OPEN_FILEDESCRIPTORS 1024
 
@@ -169,6 +288,7 @@
 #undef USHRT_MAX
 
 /* Define these if you are going to use threads */
+#undef PIKE_THREADS
 #undef _REENTRANT
 #undef _THREAD_SAFE
 
@@ -188,12 +308,21 @@
 /* Define this if your pthreads have pthread_condattr_default */
 #undef HAVE_PTHREAD_CONDATTR_DEFAULT
 
+/* Define if your pthreads have PTHREAD_MUTEX_RECURSIVE */
+#undef HAVE_PTHREAD_MUTEX_RECURSIVE
+
+/* Define if your pthreads have PTHREAD_MUTEX_RECURSIVE_NP */
+#undef HAVE_PTHREAD_MUTEX_RECURSIVE_NP
+
 /* Define this if you need to use &pthread_condattr_default in cond_init() */
 #undef HAVE_PTHREAD_CONDATTR_DEFAULT_AIX
 
 /* Define if you have the pthread_attr_setstacksize function.  */
 #undef HAVE_PTHREAD_ATTR_SETSTACKSIZE
 
+/* Define if you have the pthread_atfork function.  */
+#undef HAVE_PTHREAD_ATFORK
+
 /* Define if you have the pthread_cond_init function.  */
 #undef HAVE_PTHREAD_COND_INIT
 
@@ -215,6 +344,9 @@
 /* Define if you don't have F_SETFD, or it doesn't work */
 #undef HAVE_BROKEN_F_SETFD
 
+/* Define if your thread implementation doesn't propagate euid & egid. */
+#undef HAVE_BROKEN_LINUX_THREAD_EUID
+
 /* Define if your cpp supports K&R-style concatenation */
 #undef HAVE_KR_CONCAT
 
@@ -237,7 +369,7 @@
 #undef WITH_OOB
 
 /* Enable individual tracing of threads */
-#undef TRACE_THREADS
+#undef THREAD_TRACE
 
 /* Enable tracing of the compiler */
 #undef YYDEBUG
@@ -251,6 +383,12 @@
 /* The last argument to accept() is an ACCEPT_SIZE_T * */
 #define ACCEPT_SIZE_T	int
 
+/* Can we compile in MMX support? */
+#undef TRY_USE_MMX
+
+/* Define if you have the <sys/resource.h> header file.  */
+#undef HAVE_SYS_RESOURCE_H
+
 @BOTTOM@
 
 /* NT stuff */
@@ -259,6 +397,7 @@
 #undef HAVE_FREELIBRARY
 #undef HAVE_GETPROCADDRESS
 #undef DL_EXPORT
+#undef USE_MY_WIN32_DLOPEN
 
 /* How to set a socket non-blocking */
 #undef USE_IOCTL_FIONBIO
@@ -267,6 +406,15 @@
 #undef USE_FCNTL_FNDELAY
 #undef USE_FCNTL_O_NONBLOCK
 
+/* How well is OOB TCP working?
+ * -1 = unknown
+ *  0 = doesn't seem to be working at all
+ *  1 = very limited functionality
+ *  2 = should be working as long as you are cautious
+ *  3 = works excellently
+ */
+#define PIKE_OOB_WORKS -1
+
 /* We want to use errno later */
 #ifdef _SGI_SPROC_THREADS
 /* Magic define of _SGI_MP_SOURCE above might redefine errno below */
@@ -276,12 +424,26 @@
 #endif /* HAVE_OSERROR && !errno */
 #endif /* _SGI_SPROC_THREADS */
 
+/* This macro is only provided for compatibility with
+ * Windows PreRelease. Use ALIGNOF() instead!
+ * (Needed for va_arg().)
+ */
+#ifndef __alignof
+#define __alignof(X) ((size_t)&(((struct { char ignored_ ; X fooo_; } *)0)->fooo_))
+#endif /* __alignof */
+
 #ifdef HAVE_FUNCTION_ATTRIBUTES
 #define ATTRIBUTE(X) __attribute__ (X)
 #else
 #define ATTRIBUTE(X)
 #endif
 
+#ifdef HAVE_DECLSPEC
+#define DECLSPEC(X) __declspec(X)
+#else /* !HAVE_DECLSPEC */
+#define DECLSPEC(X)
+#endif /* HAVE_DECLSPEC */
+
 #ifndef HAVE_WORKING___FUNC__
 #ifdef HAVE_WORKING___FUNCTION__
 #define __func__	__FUNCTION__
@@ -306,4 +468,7 @@
 #endif /* HAVE_KR_CONCAT */
 #endif /* HAVE_ANSI_CONCAT */
 
+#define TOSTR(X)	#X
+#define DEFINETOSTR(X)	TOSTR(X)
+
 #endif /* MACHINE_H */
