@@ -3,8 +3,9 @@
 ||| Pike is distributed as GPL (General Public License)
 ||| See the files COPYING and DISCLAIMER for more information.
 \*/
+/**/
 #include "global.h"
-RCSID("$Id: backend.c,v 1.40 1998/11/22 11:02:33 hubbe Exp $");
+RCSID("$Id: backend.c,v 1.65 2001/07/02 20:32:54 mast Exp $");
 #include "fdlib.h"
 #include "backend.h"
 #include <errno.h>
@@ -17,7 +18,7 @@ RCSID("$Id: backend.c,v 1.40 1998/11/22 11:02:33 hubbe Exp $");
 #include <string.h>
 #include "interpret.h"
 #include "object.h"
-#include "error.h"
+#include "pike_error.h"
 #include "fd_control.h"
 #include "main.h"
 #include "callback.h"
@@ -26,14 +27,24 @@ RCSID("$Id: backend.c,v 1.40 1998/11/22 11:02:33 hubbe Exp $");
 
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
+#else
+/* BeOS socket (select etc) stuff */
+#ifdef HAVE_NET_SOCKET_H
+#include <net/socket.h>
+#endif
 #endif
-
 #include <sys/stat.h>
 
 #define SELECT_READ 1
 #define SELECT_WRITE 2
 
+/* #define POLL_DEBUG */
 
+#ifdef POLL_DEBUG
+#define IF_PD(x)	x
+#else /* !POLL_DEBUG */
+#define IF_PD(x)
+#endif /* POLL_DEBUG */
 
 struct cb_data
 {
@@ -49,7 +60,23 @@ struct fd_datum
 #endif
 };
 
-struct fd_datum fds[MAX_OPEN_FILEDESCRIPTORS];
+struct fd_datum *fds;
+PMOD_EXPORT int fds_size = 0;
+
+#define ASSURE_FDS_SIZE(X) do{while(fds_size-1 < X) grow_fds();}while(0)
+
+void grow_fds(void)
+{
+  if( !fds_size )
+    fds_size = 16;
+  fds_size *= 2;
+  fds = realloc( fds, sizeof(struct fd_datum) * fds_size );
+  if( !fds )
+    fatal("Out of memory in backend::grow_fds()\n"
+          "Tried to allocate %d fd_datum structs\n", fds_size);
+  MEMSET( fds+(fds_size/2), 0, fds_size*sizeof(struct fd_datum)/2 );
+}
+
 
 #ifndef HAVE_AND_USE_POLL
 #undef HAVE_POLL
@@ -83,6 +110,13 @@ static struct selectors selectors;
 
 /* Some constants... */
 
+/* Notes on POLLRDNORM and POLLIN:
+ *
+ * According to the AIX manual, POLLIN and POLLRDNORM are both set
+ * if there's a nonpriority message on the read queue. POLLIN is
+ * also set if the message is of 0 length.
+ */
+
 #ifndef POLLRDNORM
 #define POLLRDNORM	POLLIN
 #endif /* !POLLRDNORM */
@@ -105,6 +139,7 @@ int active_num_in_poll = 0;
 void POLL_FD_SET(int fd, short add)
 {
   int i;
+  IF_PD(fprintf(stderr, "BACKEND: POLL_FD_SET(%d, 0x%04x)\n", fd, add));
   for(i=0; i<num_in_poll; i++)
     if(poll_fds[i].fd == fd)
     {
@@ -126,6 +161,7 @@ void POLL_FD_SET(int fd, short add)
 void POLL_FD_CLR(int fd, short sub)
 {
   int i;
+  IF_PD(fprintf(stderr, "BACKEND: POLL_FD_CLR(%d, 0x%04x)\n", fd, sub));
   if(!poll_fds) return;
   for(i=0; i<num_in_poll; i++)
     if(poll_fds[i].fd == fd)
@@ -153,6 +189,8 @@ void switch_poll_set(void)
   struct pollfd *tmp = active_poll_fds;
   int sz = active_poll_fd_size;
 
+  IF_PD(fprintf(stderr, "BACKEND: switch_poll_set()\n"));
+
   active_num_in_poll = num_in_poll;
 
   if(!num_in_poll) return;
@@ -177,14 +215,14 @@ void switch_poll_set(void)
 
 
 static int max_fd;
-struct timeval current_time;
-struct timeval next_timeout;
+PMOD_EXPORT struct timeval current_time;
+PMOD_EXPORT struct timeval next_timeout;
 static int wakeup_pipe[2];
 static int may_need_wakeup=0;
 
 static struct callback_list backend_callbacks;
 
-struct callback *add_backend_callback(callback_func call,
+PMOD_EXPORT struct callback *debug_add_backend_callback(callback_func call,
 				      void *arg,
 				      callback_func free_func)
 {
@@ -200,7 +238,7 @@ static void wakeup_callback(int fd, void *foo)
 /* This is used by threaded programs and signals to wake up the
  * master 'thread'.
  */
-void wake_up_backend(void)
+PMOD_EXPORT void wake_up_backend(void)
 {
   char foo=0;
   if(may_need_wakeup)
@@ -244,15 +282,30 @@ void cleanup_backend(void)
     active_num_in_poll = 0;
   }
 #endif /* HAVE_POLL */
+#ifdef HAVE_BROKEN_F_SETFD
+  cleanup_close_on_exec();
+#endif /* HAVE_BROKEN_F_SETFD */
+  if (fds) {
+    free(fds);
+    fds = NULL;
+    fds_size = 0;
+  }
 }
 
 void set_read_callback(int fd,file_callback cb,void *data)
 {
 #ifdef HAVE_POLL
-  int was_set = (fds[fd].read.callback!=0);
+  int was_set;
+#endif
+  IF_PD(fprintf(stderr, "BACKEND: set_read_callback(%d, %p, %p)\n",
+		fd, cb, data));
+
+  ASSURE_FDS_SIZE( fd );
+#ifdef HAVE_POLL
+  was_set = (fds[fd].read.callback!=0);
 #endif
 #ifdef PIKE_DEBUG
-  if(fd<0 || fd>=MAX_OPEN_FILEDESCRIPTORS)
+  if(fd<0)
     fatal("File descriptor out of range.\n %d",fd);
 #endif
   fds[fd].read.callback=cb;
@@ -262,7 +315,7 @@ void set_read_callback(int fd,file_callback cb,void *data)
   {
 #ifdef HAVE_POLL
     if(!was_set)
-      POLL_FD_SET(fd, POLLRDNORM);
+      POLL_FD_SET(fd, POLLRDNORM|POLLIN);
 #else
     my_FD_SET(fd, &selectors.read);
 #endif
@@ -270,8 +323,13 @@ void set_read_callback(int fd,file_callback cb,void *data)
     wake_up_backend();
   }else{
 #ifdef HAVE_POLL
+#if defined(WITH_OOB)
+    if (was_set && !fds[fd].read_oob.callback)
+      POLL_FD_CLR(fd, POLLRDNORM|POLLIN);
+#else /* !WITH_OOB */
     if(was_set)
-      POLL_FD_CLR(fd, POLLRDNORM);
+      POLL_FD_CLR(fd, POLLRDNORM|POLLIN);
+#endif /* WITH_OOB */
 #else /* !HAVE_POLL */
     if(fd <= max_fd)
     {
@@ -296,10 +354,17 @@ void set_read_callback(int fd,file_callback cb,void *data)
 void set_write_callback(int fd,file_callback cb,void *data)
 {
 #ifdef HAVE_POLL
-  int was_set = (fds[fd].write.callback!=0);
+  int was_set;
+#endif
+  IF_PD(fprintf(stderr, "BACKEND: set_write_callback(%d, %p, %p)\n",
+		fd, cb, data));
+
+  ASSURE_FDS_SIZE( fd );
+#ifdef HAVE_POLL
+  was_set = (fds[fd].write.callback!=0);
 #endif
 #ifdef PIKE_DEBUG
-  if(fd<0 || fd>=MAX_OPEN_FILEDESCRIPTORS)
+  if(fd<0)
     fatal("File descriptor out of range.\n %d",fd);
 #endif
 
@@ -318,13 +383,13 @@ void set_write_callback(int fd,file_callback cb,void *data)
     wake_up_backend();
   } else {
 #ifdef HAVE_POLL
-#if defined(WITH_OOB) && (POLLWRBAND == POLLOUT)
+#if defined(WITH_OOB)
     if (was_set && !fds[fd].write_oob.callback)
       POLL_FD_CLR(fd, POLLOUT);
-#else /* POLLWRBAND != POLLOUT */
+#else /* !WITH_OOB */
     if(was_set)
       POLL_FD_CLR(fd, POLLOUT);
-#endif /* POLLWRBAND == POLLOUT */
+#endif /* WITH_OOB */
 #else /* !HAVE_POLL */
     if(fd <= max_fd)
     {
@@ -355,10 +420,17 @@ void set_write_callback(int fd,file_callback cb,void *data)
 void set_read_oob_callback(int fd,file_callback cb,void *data)
 {
 #ifdef HAVE_POLL
-  int was_set = (fds[fd].read_oob.callback!=0);
+  int was_set;
+#endif
+  IF_PD(fprintf(stderr, "BACKEND: set_read_oob_callback(%d, %p, %p)\n",
+		fd, cb, data));
+
+  ASSURE_FDS_SIZE( fd );
+#ifdef HAVE_POLL
+  was_set = (fds[fd].read_oob.callback!=0);
 #endif
 #ifdef PIKE_DEBUG
-  if(fd<0 || fd>=MAX_OPEN_FILEDESCRIPTORS)
+  if(fd<0)
     fatal("File descriptor out of range.\n %d",fd);
 #endif
   fds[fd].read_oob.callback=cb;
@@ -367,7 +439,7 @@ void set_read_oob_callback(int fd,file_callback cb,void *data)
   if(cb)
   {
 #ifdef HAVE_POLL
-    POLL_FD_SET(fd, POLLRDBAND);
+    POLL_FD_SET(fd, POLLRDBAND|POLLRDNORM|POLLIN);
 #else
     my_FD_SET(fd, &selectors.except);
 #endif
@@ -375,8 +447,15 @@ void set_read_oob_callback(int fd,file_callback cb,void *data)
     wake_up_backend();
   }else{
 #ifdef HAVE_POLL
-    if(was_set)
-      POLL_FD_CLR(fd, POLLRDBAND);
+    if(was_set) {
+      if (!fds[fd].read.callback) {
+	POLL_FD_CLR(fd, POLLRDBAND|POLLRDNORM|POLLIN);
+      } else {
+#if (POLLRDBAND != POLLRDNORM) && (POLLRDBAND != POLLIN)
+	POLL_FD_CLR(fd, POLLRDBAND);
+#endif /* (POLLRDBAND != POLLRDNORM) && (POLLRDBAND != POLLIN) */
+      }
+    }
 #else /* !HAVE_POLL */
     if(fd <= max_fd)
     {
@@ -398,10 +477,17 @@ void set_read_oob_callback(int fd,file_callback cb,void *data)
 void set_write_oob_callback(int fd,file_callback cb,void *data)
 {
 #ifdef HAVE_POLL
-  int was_set = (fds[fd].write_oob.callback!=0);
+  int was_set;
+#endif
+  IF_PD(fprintf(stderr, "BACKEND: set_write_oob_callback(%d, %p, %p)\n",
+		fd, cb, data));
+
+  ASSURE_FDS_SIZE( fd );
+#ifdef HAVE_POLL
+  was_set = (fds[fd].write_oob.callback!=0);
 #endif
 #ifdef PIKE_DEBUG
-  if(fd<0 || fd>=MAX_OPEN_FILEDESCRIPTORS)
+  if(fd<0)
     fatal("File descriptor out of range.\n %d",fd);
 #endif
 
@@ -411,7 +497,7 @@ void set_write_oob_callback(int fd,file_callback cb,void *data)
   if(cb)
   {
 #ifdef HAVE_POLL
-    POLL_FD_SET(fd,POLLWRBAND);
+    POLL_FD_SET(fd, POLLWRBAND|POLLOUT);
 #else
     my_FD_SET(fd, &selectors.write);	/* FIXME:? */
 #endif
@@ -419,13 +505,15 @@ void set_write_oob_callback(int fd,file_callback cb,void *data)
     wake_up_backend();
   }else{
 #ifdef HAVE_POLL
-#if POLLWRBAND == POLLOUT
-    if (was_set && !fds[fd].write.callback)
-      POLL_FD_CLR(fd,POLLWRBAND);
-#else /* POLLWRBAND != POLLOUT */
-    if(was_set)
-      POLL_FD_CLR(fd,POLLWRBAND);
-#endif /* POLLWRBAND == POLLOUT */
+    if(was_set) {
+      if (!fds[fd].write.callback) {
+	POLL_FD_CLR(fd, POLLWRBAND|POLLOUT);
+      } else {
+#if POLLWRBAND != POLLOUT
+	POLL_FD_CLR(fd, POLLWRBAND);
+#endif /* POLLWRBAND != POLLOUT */
+      }
+    }
 #else /* !HAVE_POLL */
     /* FIXME:? */
     if(fd <= max_fd)
@@ -447,86 +535,94 @@ void set_write_oob_callback(int fd,file_callback cb,void *data)
 }
 #endif /* WITH_OOB */
 
-file_callback query_read_callback(int fd)
+PMOD_EXPORT file_callback query_read_callback(int fd)
 {
 #ifdef PIKE_DEBUG
-  if(fd<0 || fd>=MAX_OPEN_FILEDESCRIPTORS)
+  if(fd<0)
     fatal("File descriptor out of range.\n %d",fd);
 #endif
-
+  IF_PD(fprintf(stderr, "BACKEND: query_read_callback(%d)\n", fd));
+  ASSURE_FDS_SIZE( fd );
   return fds[fd].read.callback;
 }
 
-file_callback query_write_callback(int fd)
+PMOD_EXPORT file_callback query_write_callback(int fd)
 {
 #ifdef PIKE_DEBUG
-  if(fd<0 || fd>=MAX_OPEN_FILEDESCRIPTORS)
+  if(fd<0)
     fatal("File descriptor out of range.\n %d",fd);
 #endif
-
+  IF_PD(fprintf(stderr, "BACKEND: query_write_callback(%d)\n", fd));
+  ASSURE_FDS_SIZE( fd );
   return fds[fd].write.callback;
 }
 
 #ifdef WITH_OOB
-file_callback query_read_oob_callback(int fd)
+PMOD_EXPORT file_callback query_read_oob_callback(int fd)
 {
 #ifdef PIKE_DEBUG
-  if(fd<0 || fd>=MAX_OPEN_FILEDESCRIPTORS)
+  if(fd<0)
     fatal("File descriptor out of range.\n %d",fd);
 #endif
-
+  IF_PD(fprintf(stderr, "BACKEND: query_read_oob_callback(%d)\n", fd));
+  ASSURE_FDS_SIZE( fd );
   return fds[fd].read_oob.callback;
 }
 
-file_callback query_write_oob_callback(int fd)
+PMOD_EXPORT file_callback query_write_oob_callback(int fd)
 {
 #ifdef PIKE_DEBUG
-  if(fd<0 || fd>=MAX_OPEN_FILEDESCRIPTORS)
+  if(fd<0)
     fatal("File descriptor out of range.\n %d",fd);
 #endif
-
+  IF_PD(fprintf(stderr, "BACKEND: query_write_oob_callback(%d)\n", fd));
+  ASSURE_FDS_SIZE( fd );
   return fds[fd].write_oob.callback;
 }
 #endif /* WITH_OOB */
 
-void *query_read_callback_data(int fd)
+PMOD_EXPORT void *query_read_callback_data(int fd)
 {
 #ifdef PIKE_DEBUG
-  if(fd<0 || fd>=MAX_OPEN_FILEDESCRIPTORS)
+  if(fd<0)
     fatal("File descriptor out of range.\n %d",fd);
 #endif
-
+  IF_PD(fprintf(stderr, "BACKEND: query_read_callback_data(%d)\n", fd));
+  ASSURE_FDS_SIZE( fd );
   return fds[fd].read.data;
 }
 
-void *query_write_callback_data(int fd)
+PMOD_EXPORT void *query_write_callback_data(int fd)
 {
 #ifdef PIKE_DEBUG
-  if(fd<0 || fd>=MAX_OPEN_FILEDESCRIPTORS)
+  if(fd<0)
     fatal("File descriptor out of range.\n %d",fd);
 #endif
-
+  IF_PD(fprintf(stderr, "BACKEND: query_write_callback_data(%d)\n", fd));
+  ASSURE_FDS_SIZE( fd );
   return fds[fd].write.data;
 }
 
 #ifdef WITH_OOB
-void *query_read_oob_callback_data(int fd)
+PMOD_EXPORT void *query_read_oob_callback_data(int fd)
 {
 #ifdef PIKE_DEBUG
-  if(fd<0 || fd>=MAX_OPEN_FILEDESCRIPTORS)
+  if(fd<0)
     fatal("File descriptor out of range.\n %d",fd);
 #endif
-
+  IF_PD(fprintf(stderr, "BACKEND: query_read_oob_callback_data(%d)\n", fd));
+  ASSURE_FDS_SIZE( fd );
   return fds[fd].read_oob.data;
 }
 
-void *query_write_oob_callback_data(int fd)
+PMOD_EXPORT void *query_write_oob_callback_data(int fd)
 {
 #ifdef PIKE_DEBUG
-  if(fd<0 || fd>=MAX_OPEN_FILEDESCRIPTORS)
+  if(fd<0)
     fatal("File descriptor out of range.\n %d",fd);
 #endif
-
+  IF_PD(fprintf(stderr, "BACKEND: query_write_oob_callback_data(%d)\n", fd));
+  ASSURE_FDS_SIZE( fd );
   return fds[fd].write_oob.data;
 }
 #endif /* WITH_OOB */
@@ -545,13 +641,13 @@ void do_debug(void)
   extern void check_all_objects(void);
   extern void verify_shared_strings_tables(void);
   extern void slow_check_stack(void);
-  extern void do_gc(void);
+  extern int do_gc(void);
 
   slow_check_stack();
   check_all_arrays();
   check_all_mappings();
   check_all_programs();
-  verify_all_objects();
+  check_all_objects();
   verify_shared_strings_tables();
 
   call_callback(& do_debug_callbacks, 0);
@@ -598,10 +694,10 @@ void do_debug(void)
       switch(errno)
       {
        case EBADF:
-	 fatal("Backend filedescriptor %d is bad.\n",poll_fds[e].fd);
+	 fatal("Backend filedescriptor %ld is bad.\n", (long)poll_fds[e].fd);
 	 break;
        case ENOENT:
-	 fatal("Backend filedescriptor %d is not.\n",poll_fds[e].fd);
+	 fatal("Backend filedescriptor %ld is not.\n", (long)poll_fds[e].fd);
 	 break;
       }
     }
@@ -626,12 +722,9 @@ void backend(void)
 
   if(SETJMP(back))
   {
-    ONERROR tmp;
-    SET_ONERROR(tmp,exit_on_error,"Error in handle_error in master object!");
-    assign_svalue_no_free(sp++, & throw_value);
-    APPLY_MASTER("handle_error", 1);
-    pop_stack();
-    UNSET_ONERROR(tmp);
+    call_handle_error();
+    throw_value.subtype=NUMBER_UNDEFINED;
+    throw_value.u.integer=0;
   }
 
   while(first_object)
@@ -661,7 +754,13 @@ void backend(void)
     if(d_flag > 1) do_debug();
 #endif
 
+#ifndef OWN_GETHRTIME
     GETTIMEOFDAY(&current_time);
+#else
+    /* good place to run the gethrtime-conversion update
+       since we have to run gettimeofday anyway /Mirar */
+    own_gethrtime_update(&current_time);
+#endif
 
     if(my_timercmp(&next_timeout, > , &current_time))
     {
@@ -675,7 +774,10 @@ void backend(void)
 #ifdef HAVE_POLL
     {
       int msec = (next_timeout.tv_sec*1000) + next_timeout.tv_usec/1000;
+      IF_PD(fprintf(stderr, "BACKEND: poll(%p, %d, %ld)...",
+		    active_poll_fds, active_num_in_poll, msec));
       i = poll(active_poll_fds, active_num_in_poll, msec);
+      IF_PD(fprintf(stderr, " => %d\n", i));
     }
 #else
     /* FIXME: OOB? */
@@ -753,6 +855,9 @@ void backend(void)
 #ifdef WITH_OOB
 	if ((active_poll_fds[i].revents & POLLRDBAND) &&
 	    fds[fd].read_oob.callback) {
+	  IF_PD(fprintf(stderr, "BACKEND: POLLRDBAND\n"));
+	  IF_PD(fprintf(stderr, "BACKEND: read_oob_callback(%d, %p)\n",
+			fd, fds[fd].read_oob.data));
 	  (*(fds[fd].read_oob.callback))(fd, fds[fd].read_oob.data);
 #ifdef PIKE_DEBUG
 	  handled = 1;
@@ -768,8 +873,15 @@ void backend(void)
 	    fprintf(stderr, "Got POLLERR on fd %d\n", i);
 	  }
 #endif /* PIKE_DEBUG */
+	  IF_PD(fprintf(stderr, "BACKEND: POLLHUP | POLLERR\n"));
 	  if (fds[fd].read.callback) {
+	    IF_PD(fprintf(stderr, "BACKEND: read_callback(%d, %p)\n",
+			  fd, fds[fd].read.data));
 	    (*(fds[fd].read.callback))(fd,fds[fd].read.data);
+	  } else if (fds[fd].write.callback) {
+	    IF_PD(fprintf(stderr, "BACKEND: write_callback(%d, %p)\n",
+			  fd, fds[fd].write.data));
+	    (*(fds[fd].write.callback))(fd, fds[fd].write.data);
 	  }
 	  /* We don't want to keep this fd anymore. */
 	  POLL_FD_CLR(fd, ~0);
@@ -778,11 +890,14 @@ void backend(void)
 #endif /* PIKE_DEBUG */
 	}
 
-	if(active_poll_fds[i].revents & POLLRDNORM) {
+	if(active_poll_fds[i].revents & (POLLRDNORM|POLLIN)) {
+	  IF_PD(fprintf(stderr, "BACKEND: POLLRDNORM|POLLIN\n"));
 	  if (fds[fd].read.callback) {
+	    IF_PD(fprintf(stderr, "BACKEND: read_callback(%d, %p)\n",
+			  fd, fds[fd].read.data));
 	    (*(fds[fd].read.callback))(fd,fds[fd].read.data);
 	  } else {
-	    POLL_FD_CLR(fd, POLLRDNORM);
+	    POLL_FD_CLR(fd, POLLRDNORM|POLLIN);
 	  }
 #ifdef PIKE_DEBUG
 	  handled = 1;
@@ -792,6 +907,9 @@ void backend(void)
 #ifdef WITH_OOB
 	if ((active_poll_fds[i].revents & POLLWRBAND) &&
 	    fds[fd].write_oob.callback) {
+	  IF_PD(fprintf(stderr, "BACKEND: POLLWRBAND\n"));
+	  IF_PD(fprintf(stderr, "BACKEND: write_oob_callback(%d, %p)\n",
+			fd, fds[fd].write_oob.data));
 	  (*(fds[fd].write_oob.callback))(fd, fds[fd].write_oob.data);
 #ifdef PIKE_DEBUG
 	  handled = 1;
@@ -800,7 +918,10 @@ void backend(void)
 #endif /* WITH_OOB */
 
 	if(active_poll_fds[i].revents & POLLOUT) {
+	  IF_PD(fprintf(stderr, "BACKEND: POLLOUT\n"));
 	  if (fds[fd].write.callback) {
+	    IF_PD(fprintf(stderr, "BACKEND: write_callback(%d, %p)\n",
+			  fd, fds[fd].write.data));
 	    (*(fds[fd].write.callback))(fd, fds[fd].write.data);
 	  } else {
 	    POLL_FD_CLR(fd, POLLOUT);
@@ -812,9 +933,10 @@ void backend(void)
 #ifdef PIKE_DEBUG
 	num_active -= handled;
 	if (!handled && active_poll_fds[i].revents) {
-	  fprintf(stderr, "BACKEND: fd %d has revents 0x%08x, "
-		  "but hasn't been handled.\n", active_poll_fds[i].fd,
-		  active_poll_fds[i].revents);
+	  fprintf(stderr, "BACKEND: fd %ld has revents 0x%08lx, "
+		  "but hasn't been handled.\n",
+		  (long)active_poll_fds[i].fd,
+		  (long)active_poll_fds[i].revents);
 	}
 #endif /* PIKE_DEBUG */
       }
@@ -823,9 +945,10 @@ void backend(void)
 	fprintf(stderr, "BACKEND: %d more active fds than were handled.\n",
 		num_active);
 	for(i=0; i<active_num_in_poll; i++) {
-	  fprintf(stderr, "BACKEND: fd %d, events 0x%08x, revents 0x%08x\n",
-		  active_poll_fds[i].fd, active_poll_fds[i].events,
-		  active_poll_fds[i].revents);
+	  fprintf(stderr, "BACKEND: fd %ld, events 0x%08lx, revents 0x%08lx\n",
+		  (long)active_poll_fds[i].fd,
+		  (long)active_poll_fds[i].events,
+		  (long)active_poll_fds[i].revents);
 	}
       }
 #endif /* PIKE_DEBUG */
@@ -885,7 +1008,7 @@ void backend(void)
 	    case EBADF:
 	    {
 	      int i;
-	      for(i=0;i<MAX_OPEN_FILEDESCRIPTORS;i++)
+	      for(i=0;i<fds_size;i++)
 	      {
 		if(!my_FD_ISSET(i, &selectors.read) &&
 		   !my_FD_ISSET(i, &selectors.write)
@@ -940,6 +1063,7 @@ void backend(void)
 	    }
 	  }
 #ifdef _REENTRANT
+	  /* FIXME: Extra stderr messages should not be allowed.../Hubbe */
 	  write_to_stderr("Bad filedescriptor to select().\n"
 			  "fd closed in another thread?\n", 62);
 #else /* !_REENTRANT */
@@ -951,21 +1075,22 @@ void backend(void)
 
       }
     }
-    call_callback(& backend_callbacks, (void *)1);
+    call_callback(& backend_callbacks, (void *)(ptrdiff_t)1);
   }
 
   UNSETJMP(back);
 }
 
-int write_to_stderr(char *a, INT32 len)
+PMOD_EXPORT int write_to_stderr(char *a, size_t len)
 {
 #ifdef __NT__
-  int e;
+  size_t e;
   for(e=0;e<len;e++)
     putc(a[e],stderr);
 #else
   int nonblock=0;
-  INT32 pos, tmp;
+  size_t pos;
+  int tmp;
 
   if(!len) return 1;
 
