@@ -5,28 +5,38 @@
 \*/
 
 /*
- * $Id: main.h,v 1.9 1998/04/13 14:30:52 grubba Exp $
+ * $Id: main.h,v 1.15 2000/12/16 05:24:40 marcus Exp $
  */
 #ifndef MAIN_H
 #define MAIN_H
 
 #include "callback.h"
 
-extern int d_flag, t_flag, a_flag, l_flag, c_flag, p_flag, debug_options;
-extern int default_t_flag;
+PMOD_EXPORT extern int d_flag, t_flag, a_flag, l_flag, c_flag, p_flag;
+PMOD_EXPORT extern int debug_options, runtime_options;
+PMOD_EXPORT extern int default_t_flag;
 
+#ifdef TRY_USE_MMX
+extern int try_use_mmx;
+#endif
+
+/* Debug options */
 #define DEBUG_SIGNALS 1
 #define NO_TAILRECURSION 2
 
+/* Runtime options */
+#define RUNTIME_CHECK_TYPES  1
+#define RUNTIME_STRICT_TYPES 2
+
 /* Prototypes begin here */
-struct callback *add_post_master_callback(callback_func call,
+PMOD_EXPORT struct callback *add_post_master_callback(callback_func call,
 					  void *arg,
 					  callback_func free_func);
-struct callback *add_exit_callback(callback_func call,
+PMOD_EXPORT struct callback *add_exit_callback(callback_func call,
 				   void *arg,
 				   callback_func free_func);
 int dbm_main(int argc, char **argv);
-void do_exit(int num) ATTRIBUTE((noreturn));
+DECLSPEC(noreturn) void pike_do_exit(int num) ATTRIBUTE((noreturn));
 void low_init_main(void);
 void exit_main(void);
 void init_main(void);
